from django.test import TestCase

from text.models import Text


class TestText(TestCase):
    def test_render(self):
        t = Text(body='# hello', type=Text.TYPE_MARKDOWN)
        self.assertEqual(t.render(), '<h1>hello</h1>')
        t = Text(body='# hello')
        self.assertEqual(t.render(), '# hello')

    def test_text_id(self):
        t = Text(name='hello', language='sv')
        self.assertEqual(t.text_id, 'hello_sv')



import os

TEMPLATE_DEBUG = True

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:'
    }
}

INSTALLED_APPS = (
    'text',
)

TEMPLATE_DIRS = (
    os.path.join(os.path.dirname(__file__), 'text', 'templates'),
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.request',
)

MIDDLEWARE_CLASSES = (
    'text.middleware.TextMiddleware',
    'text.middleware.ToolbarMiddleware',
)

ROOT_URLCONF = 'text.urls'





# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('text', '0003_text_language'),
    ]

    operations = [
        migrations.AlterField(
            model_name='text',
            name='type',
            field=models.IntegerField(default=1, choices=[(0, b'Text'), (1, b'Markdown')]),
            preserve_default=True,
        ),
    ]



from django.http import JsonResponse, Http404, HttpResponse
from django.views.generic import DetailView, UpdateView

from .models import Text
from .forms import TextForm
from .conf import settings
from .utils import access_toolbar


class TextView(DetailView):
    model = Text

    def get_object(self, queryset=None):
        slug = self.kwargs.get('text_slug', None)
        parts = slug.split('_')
        language = parts[-1]
        name = '_'.join(parts[:-1])
        if queryset is None:
            queryset = self.get_queryset()
        queryset = queryset.filter(name=name, language=language)
        try:
            # Get the single item from the filtered queryset
            obj = queryset.get()
        except queryset.model.DoesNotExist:
            raise Http404("Found no text with that id")
        return obj

    def get(self, request, *args, **kwargs):
        if not access_toolbar(request):
            raise Http404()
        self.object = self.get_object()
        data = {
            'id': self.object.id,
            'body': self.object.body,
            'type': self.object.type,
            'language': self.object.language,
            'render': self.object.render(),
            'name': self.object.name,
        }
        return JsonResponse(data=data)


class TextUpdateView(UpdateView):
    model = Text
    form_class = TextForm
    pk_url_kwarg = 'text_id'
    form_prefix = settings.TOOLBAR_FORM_PREFIX

    def get_form_kwargs(self):
        kwargs = super(TextUpdateView, self).get_form_kwargs()
        if self.form_prefix:
            kwargs.update({'prefix': self.form_prefix})
        return kwargs

    def post(self, request, *args, **kwargs):
        if not access_toolbar(request):
            raise Http404()
        self.object = self.get_object()
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        if form.is_valid():
            self.object = form.save()
            return HttpResponse(status=204)
        else:
            return JsonResponse({'errors': form.errors, 'success': False})



from django.conf.urls import patterns, url

from .views import TextView, TextUpdateView


urlpatterns = patterns(
    '',
    url(r'^text/(?P<text_slug>\w+)/$', TextView.as_view(), name='text'),
    url(r'^update_text/(?P<text_id>\d+)/$', TextUpdateView.as_view(), name='update_text'),
)



from django.test import TestCase
from django.template import FilterExpression, Parser
from django.http import HttpRequest

from text.templatetags.editable import TextNode
from text.conf import settings


class TestTextNode(TestCase):
    def fe(self, name):
        fe = FilterExpression('"{0}"'.format(name), Parser([]))
        return fe

    def test_get_wrapped_placeholder(self):
        name = 'name_of_text_node'
        fe = self.fe(name)
        settings.TOOLBAR_INSTANT_UPDATE = True
        placeholder = TextNode(fe).get_placeholder({})
        expected = '<span data-text-name="name_of_text_node" class="dj_text_inline_wrapper">{{ text_placeholder_name_of_text_node }}</span>'
        self.assertEqual(placeholder, expected)

    def test_get_placeholder(self):
        name = 'name_of_text_node'
        fe = self.fe(name)
        settings.TOOLBAR_INSTANT_UPDATE = False
        placeholder = TextNode(fe).get_placeholder({})
        expected = '{{ text_placeholder_name_of_text_node }}'
        self.assertEqual(placeholder, expected)

    def test_set_default(self):
        name = 'name_of_text_node'
        content = 'test node content'
        context = {'request': HttpRequest()}
        fe = self.fe(name)
        t = TextNode(fe)
        t.set_default(name, context, content)
        self.assertEqual(context['request'].text_default_register[name], content)

    def test_set_type(self):
        name = 'name_of_text_node'
        type = 'html'
        context = {'request': HttpRequest()}
        fe = self.fe(name)
        t = TextNode(fe)
        t.set_type(name, context, type)
        self.assertEqual(context['request'].text_type_register[name], type)

    def test_register(self):
        name = 'name_of_text_node'
        context = {'request': HttpRequest()}
        fe = self.fe(name)
        t = TextNode(fe)
        t.register(name, context)
        self.assertIn(name, context['request'].text_register)

    def test_init_logic(self):
        name = self.fe('name_of_text_node')
        t = TextNode(name)
        self.assertEqual(t.text_name, t.default_text)
        default_text = self.fe('hejhej!')
        t = TextNode(name, default_text)
        self.assertEqual(t.default_text, default_text)



from django import forms

from .models import Text
from .widgets import HTMLEditorWidget


class TextForm(forms.ModelForm):
    class Meta:
        model = Text
        exclude = ['language']

    def __init__(self, *args, **kwargs):
        super(TextForm, self).__init__(*args, **kwargs)
        self.fields['name'].widget = forms.HiddenInput()
        self.fields['body'].widget = HTMLEditorWidget()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('text', '0005_text_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='text',
            name='type_char',
            field=models.CharField(default=b'markdown', max_length=20, choices=[(b'text', b'Text'), (b'markdown', b'Markdown')]),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('text', '0002_text_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='text',
            name='language',
            field=models.CharField(default=settings.LANGUAGE_CODE, max_length=5, choices=settings.LANGUAGES),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='text',
            name='name',
            field=models.CharField(max_length=50, db_index=True),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='text',
            unique_together=set([('name', 'language')]),
        ),
        migrations.AlterIndexTogether(
            name='text',
            index_together=set([('name', 'language')]),
        ),
    ]



from .conf import settings


def access_toolbar(request):
    if not settings.TOOLBAR_ENABLED:
        return False
    user = request.user
    return (user.is_authenticated() and user.is_active and user.is_staff and
            user.has_perm('text.change_text'))



from django.db import models
from django.conf import settings
from django.utils.safestring import mark_safe

import markdown


class Text(models.Model):
    TYPE_TEXT = 'text'
    TYPE_MARKDOWN = 'markdown'
    TYPE_HTML = 'html'
    TYPES = (
        (TYPE_TEXT, 'Text'),
        (TYPE_MARKDOWN, 'Markdown'),
        (TYPE_HTML, 'HTML'),
    )

    name = models.CharField(max_length=50, db_index=True)
    body = models.TextField()
    type = models.CharField(
        choices=TYPES,
        blank=False,
        default=TYPE_TEXT,
        max_length=20)
    language = models.CharField(
        choices=settings.LANGUAGES,
        max_length=5,
        default=settings.LANGUAGE_CODE)

    class Meta:
        unique_together = ('name', 'language', )
        index_together = ['name', 'language', ]
        ordering = ('name', 'language', )

    def __unicode__(self):
        return self.text_id

    def render_markdown(self, text):
        return markdown.markdown(text, output_format='html5')

    def render_html(self, text):
        return text

    def render_text(self, text):
        return text

    def render(self):
        render = getattr(self, 'render_{0}'.format(self.type))
        return mark_safe(render(self.body))

    @property
    def text_id(self):
        return "%s_%s" % (self.name, self.language)



from django.test import TestCase
from django.http import HttpRequest
from django.template import Template, FilterExpression, Parser, Context
from django.template.response import SimpleTemplateResponse

from mock import patch

from text.middleware import build_context, create_text, TextMiddleware
from text.models import Text
from text.conf import settings
from text.templatetags.editable import TextNode


class TestBuildContext(TestCase):
    def test_use_default(self):
        texts = []
        defaults = {'a_text_node': '<p>wooot</p>'}
        types = {'a_text_node': 'html'}
        c = build_context(texts, defaults, types)
        # test return value
        self.assertEqual(c['text_placeholder_a_text_node'], defaults['a_text_node'])
        # test model integration
        t = Text.objects.get(name='a_text_node')
        self.assertEqual(t.render(), defaults['a_text_node'])
        self.assertEqual(t.type, types['a_text_node'])

    @patch('text.middleware.create_text')
    def test_use_db(self, create_text):
        t = Text(name='my_text_node', type='markdown', body='# Hello')
        t.save()
        texts = [t]
        defaults = {'my_text_node': '<p>LOL</p>'}
        types = {'my_text_node': 'markdown'}
        c = build_context(texts, defaults, types)
        self.assertFalse(create_text.called)
        self.assertEqual(c['text_placeholder_my_text_node'], t.render())


class TestCreateText(TestCase):
    def test_use_default_type(self):
        name = 'my_text'
        body = 'my text is so awesome'
        t = create_text(name, body, None)
        self.assertEqual(t.type, Text.TYPE_TEXT)

    def test_save(self):
        name = 'my_text'
        body = 'my text is so awesome'
        text_type = 'html'
        settings.AUTOPOPULATE_TEXT = True
        t = create_text(name, body, text_type)
        tdb = Text.objects.get(name=name)
        self.assertEqual(t.type, text_type)
        self.assertEqual(tdb.type, text_type)
        self.assertEqual(t.name, name)
        self.assertEqual(tdb.name, name)
        self.assertEqual(t.body, body)
        self.assertEqual(tdb.body, body)

    def test_no_autopopulate(self):
        settings.AUTOPOPULATE_TEXT = False
        create_text('a_name', 'a body', Text.TYPE_HTML)
        self.assertEqual(Text.objects.count(), 0)


class TestTextMiddleware(TestCase):
    def fe(self, name):
        fe = FilterExpression('"{0}"'.format(name), Parser([]))
        return fe

    def process_template_response(self, name, default=''):
        settings.TOOLBAR_INSTANT_UPDATE = False
        request = HttpRequest()
        context = Context({'request': request})
        node = TextNode(self.fe(name), self.fe(default)).render(context)
        template = Template(node)
        response = SimpleTemplateResponse(template, context)
        response.content = node
        mw = TextMiddleware()
        return mw.process_template_response(request, response).render()

    def test_replace_with_default(self):
        content = "some test content"
        rendered = self.process_template_response('node', content)
        self.assertEqual(rendered.content, content)

    def test_replace_no_default(self):
        name = 'test_node'
        rendered = self.process_template_response(name)
        self.assertEqual(rendered.content, name)

    def test_replace_db(self):
        text = Text(name='db_node', body='my awesome text', type=Text.TYPE_TEXT)
        text.save()
        rendered = self.process_template_response(text.name, default='this is the default')
        self.assertEqual(rendered.content, text.render())



from django import forms
from django.utils.html import format_html
from django.forms.utils import flatatt
from django.utils.encoding import force_text

from text.models import Text


MARKDOWN_TEMPLATE = u"""
<textarea{0}>\r\n{1}</textarea>
<div class="djtext_html_editor">{2}</div>
"""


class HTMLEditorWidget(forms.widgets.Textarea):
    class Media:
        css = {
            'all': (
                'text/bundle/medium-editor-3.0.0/css/medium-editor.min.css',
                'text/bundle/medium-editor-3.0.0/css/themes/default.min.css',
                'text/css/html-widget.css',
                '//fonts.googleapis.com/css?family=Lora:400,700,400italic,700italic&subset=latin,latin-ext',
            )
        }
        js = (
            'text/bundle/medium-editor-3.0.0/js/medium-editor.min.js',
            'text/js/html-widget.js',
        )

    def __init__(self, attrs=None):
        default_attrs = {'cols': '40', 'rows': '10', 'class': 'djtext_editor_input', }
        if attrs:
            default_attrs.update(attrs)
        super(HTMLEditorWidget, self).__init__(default_attrs)

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        final_attrs = self.build_attrs(attrs, name=name)
        t = Text(type=Text.TYPE_HTML, body=value)
        rendered = t.render()
        return format_html(
            MARKDOWN_TEMPLATE,
            flatatt(final_attrs),
            force_text(value),
            rendered)



from django.conf import settings as django_settings


class Conf(object):
    AUTOPOPULATE_TEXT = getattr(django_settings, 'AUTOPOPULATE_TEXT', True)
    TOOLBAR_FORM_PREFIX = getattr(django_settings, 'TEXT_TOOLBAR_FORM_PREFIX', 'djtext_form')
    TOOLBAR_ENABLED = getattr(django_settings, 'TEXT_TOOLBAR_ENABLED', True)
    TOOLBAR_INSTANT_UPDATE = getattr(django_settings, 'TEXT_TOOLBAR_INSTANT_UPDATE', True)
    INLINE_WRAPPER = getattr(django_settings, 'TEXT_INLINE_WRAPPER', (
        '<span data-text-name="{0}" class="{1}">', '</span>'))
    INLINE_WRAPPER_CLASS = getattr(django_settings, 'TEXT_INLINE_WRAPPER_CLASS', 'dj_text_inline_wrapper')

settings = Conf()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def populate_type_char(apps, schema_editor):
    Text = apps.get_model(b'text', b'Text')
    types = {
        0: b'text',
        1: b'markdown',
    }
    for text in Text.objects.all().iterator():
        text.type_char = types[text.type]
        text.save()


class Migration(migrations.Migration):

    dependencies = [
        ('text', '0006_text_type_char'),
    ]

    operations = [
        migrations.RunPython(populate_type_char),
    ]



#!/usr/bin/env python

from distutils.core import setup
from setuptools import find_packages

from text import __version__


setup(
    name='django-text',
    description='Intuitive text editing for humans using Django.',
    version=__version__,
    long_description=open('README.rst').read(),
    author='Anton Agestam',
    author_email='msn@antonagestam.se',
    packages=find_packages(),
    url='https://github.com/antonagestam/django-text/',
    license='The MIT License (MIT)',
    include_package_data=True,
    zip_safe=False,  # because we're including static files
    install_requires=['Django>=1.7.4', 'Markdown>=2.6', ],
    classifiers=['Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: 3']
)



import re

from django.template.response import SimpleTemplateResponse
from django.template import Template, Context, RequestContext
from django.template.loader import get_template
from django.utils.translation import get_language
from django.utils.encoding import force_text
from django.conf import settings as django_settings

from .conf import settings
from .models import Text
from .forms import TextForm
from .utils import access_toolbar


def build_context(texts, defaults, types):
    placeholder = "text_placeholder_{0}"
    context = {placeholder.format(t.name): t.render() for t in texts}
    for name, text in defaults.iteritems():
        pname = placeholder.format(name)
        if pname not in context:
            context[pname] = create_text(name, text, types[name]).render()
    return context


def create_text(name, body, text_type):
    if text_type is None or text_type not in dict(Text.TYPES).keys():
        text_type = Text._meta.get_field('type').get_default()

    language = get_language()
    text = Text(
        name=name,
        body=body,
        language=language,
        type=text_type)
    if settings.AUTOPOPULATE_TEXT:
        text.save()
    return text


class TextMiddleware(object):
    def process_template_response(self, request, response):
        template = Template(response.render().content)
        if not hasattr(request, 'text_register'):
            return response
        language = get_language()
        texts = Text.objects.filter(
            name__in=request.text_register,
            language=language)
        defaults = getattr(request, 'text_default_register', {})
        types = getattr(request, 'text_type_register', {})
        context = Context(build_context(texts, defaults, types))
        return SimpleTemplateResponse(template, context)


class ToolbarMiddleware(object):
    def process_response(self, request, response):
        texts = getattr(request, 'text_register', None)
        if request.is_ajax() or not texts or not access_toolbar(request):
            return response
        toolbar = get_template('text/text_toolbar.html')
        form = TextForm(prefix=settings.TOOLBAR_FORM_PREFIX)
        context = RequestContext(request, {
            'texts': texts,
            'language': get_language(),
            'form': form,
            'inline_wrapper_class': settings.INLINE_WRAPPER_CLASS,
        })
        insert_before = '</body>'
        pattern = re.escape(insert_before)
        content = force_text(
            response.content, encoding=django_settings.DEFAULT_CHARSET)
        bits = re.split(pattern, content, flags=re.IGNORECASE)

        if len(bits) > 1:
            bits[-2] += toolbar.render(context)
            response.content = insert_before.join(bits)
            if response.get('Content-Length', None):
                response['Content-Length'] = len(response.content)
        return response



from django.contrib import admin
from django.db import models

from .models import Text
from .widgets import HTMLEditorWidget


class TextAdmin(admin.ModelAdmin):
    list_display = ('name', 'language', 'type', )
    search_fields = ('name', )
    formfield_overrides = {
        models.TextField: {'widget': HTMLEditorWidget},
    }


admin.site.register(Text, TextAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('text', '0004_text_default_type'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='text',
            options={'ordering': ('name', 'language')},
        ),
    ]



from django import template

from text.models import Text
from text.conf import settings

register = template.Library()


class TextNode(template.Node):
    def __init__(self, text_name, default_text='', text_type=Text.TYPE_TEXT):
        self.text_name = text_name
        if not isinstance(default_text, template.FilterExpression):
            default_text = template.FilterExpression(default_text, template.Parser([]))
        if default_text.token.replace('"', '').replace("'", '') == '' or not default_text:
            default_text = text_name
        self.default_text = default_text
        self.type = text_type
        if not isinstance(self.type, template.FilterExpression):
            self.type = template.FilterExpression(self.type, template.Parser([]))

    def resolved_text_name(self, context):
        text_name = self.text_name.resolve(context)
        if not text_name or text_name.strip() == '':
            raise template.TemplateSyntaxError("Invalid text node name")
        return text_name

    def set_default(self, text_name, context, content):
        if not hasattr(context['request'], 'text_default_register'):
            context['request'].text_default_register = {}
        context['request'].text_default_register[text_name] = content.strip()

    def set_type(self, text_name, context, text_type):
        if not hasattr(context['request'], 'text_type_register'):
            context['request'].text_type_register = {}
        context['request'].text_type_register[text_name] = text_type

    def register(self, text_name, context):
        if not hasattr(context['request'], 'text_register'):
            context['request'].text_register = []
        context['request'].text_register.append(text_name)

    def get_placeholder(self, context):
        placeholder = "{{ text_placeholder_%s }}"
        text_name = self.resolved_text_name(context)
        if settings.TOOLBAR_INSTANT_UPDATE:
            before, after = settings.INLINE_WRAPPER
            before = before.format(text_name, settings.INLINE_WRAPPER_CLASS)
            placeholder = ''.join((before, placeholder, after))
        return placeholder % text_name

    def render(self, context):
        text_name = self.resolved_text_name(context)
        self.register(text_name, context)
        self.set_default(text_name, context, self.default_text.resolve(context))
        self.set_type(text_name, context, self.type.resolve(context))
        return self.get_placeholder(context)


class BlockTextNode(TextNode):
    def __init__(self, nodelist, *args, **kwargs):
        super(BlockTextNode, self).__init__(*args, **kwargs)
        self.nodelist = nodelist

    def render(self, context):
        text_name = self.resolved_text_name(context)
        self.register(text_name, context)
        self.set_default(text_name, context, self.nodelist.render(context))
        self.set_type(text_name, context, self.type.resolve(context))
        return self.get_placeholder(context)


@register.tag(name='editable')
def editable(parser, token):
    bits = token.split_contents()
    text_name = bits[1]
    try:
        default = bits[2]
        text_type = bits[3]
    except IndexError:
        default = text_name
        text_type = Text.TYPE_TEXT
    default = parser.compile_filter(default)
    text_name = parser.compile_filter(text_name)
    text_type = parser.compile_filter(text_type)
    return TextNode(text_name, default, text_type)


@register.tag(name='blockeditable')
def blockeditable(parser, token):
    nodelist = parser.parse(('endblockeditable', ))
    parser.delete_first_token()
    bits = token.split_contents()
    text_name = parser.compile_filter(bits[1])
    try:
        text_type = bits[2]
    except IndexError:
        text_type = Text.TYPE_TEXT
    text_type = parser.compile_filter(text_type)
    return BlockTextNode(nodelist, text_name, text_type=text_type)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('text', '0007_type_int_to_char'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='text',
            name='type',
        ),
    ]
